

/**
 * gpod_feed_parser_from_file()
 * @self: A #GpodFeedParser
 * @file: The file to read from
 *
 * Reads a file and returns an xml document.
 *
 * Returns: (xmlDoc *): The xml document
 */
xmlDoc *
gpod_feed_parser_from_file2 (GpodFeedParser *self,
                             gchar          *filename)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_val_if_fail (filename != NULL, NULL);

    FILE *fp;
    xmlParserCtxt *ctxt;
    gchar buffer[BUFFER_SIZE];
    gint bytes_read = 0;

    /* Check file */
    if (!g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
    {
        g_warning ("Unable to open file to parse, either does not exists or not a file.");
        return NULL;
    }

    /* Open file to read */
    fp = g_fopen (filename, "r");

    if (fp == NULL)
    {
        g_warning ("Unable to open to parse.");
        return NULL;
    }

    /* Create new xml parser */
    ctxt = xmlCreatePushParserCtxt (NULL, NULL, buffer, bytes_read, filename);

    if (ctxt == NULL)
    {
        g_warning ("Failed to create parser context.");
        return NULL;
    }

    /* Read file ny chunks */
    while ((bytes_read = fread (buffer, 1, BUFFER_SIZE, fp)) > 0)
    {
        xmlParseChunk (ctxt, buffer, bytes_read, 0);
        gchar *out = g_strndup (buffer, bytes_read);
        g_print ("%s", out);
    }

    xmlParseChunk (ctxt, buffer, 0, 1);

    fclose (fp);

    if (!ctxt->wellFormed)
    {
        g_warning ("Failed to parse file: \"%s\"", filename);
        xmlFreeParserCtxt (ctxt);
        return NULL;
    }

    xmlFreeParserCtxt (ctxt);

    return ctxt->myDoc;
}

GInputStream *
gpod_feed_parser_real_download_data_from_url (GpodFeedParser *self,
                                              gchar           *url)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_val_if_fail (url != NULL, NULL);

    g_debug ("Downloading feed from url \"%s\"", url);

    SoupSession *session;
    GError *error = NULL;
    SoupRequest *request;
    GInputStream *feed;

    session = soup_session_new ();

    request = soup_session_request (session, url, &error);

    if (error)
    {
        g_warning ("Unable to download feed from url \"%s\" because: %s",
                   url, error->message);

        g_error_free (error);

        return NULL;
    }

    feed = soup_request_send (request, NULL, &error);

    if (error)
    {
        g_warning ("Unable to retrieve stream feed from url \"%s\" because: %s",
                   url, error->message);

        g_error_free (error);

        return NULL;
    }

    return feed;
}

void
gpod_feed_parser_real_save_data_from_url (GpodFeedParser *self,
                                          gchar          *url,
                                          gchar          *local_file_uri)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_if_fail (url != NULL);
    g_return_if_fail (local_file_uri != NULL);

    g_debug ("Saving feed from url \"%s\" to \"%s\"", url, local_file_uri);

    GInputStream *stream;
    gssize bytes_read;
    gchar buffer[1024];
    GError *error = NULL;
    gboolean retval;
/*
    GFile *file;
    GFileIOStream *file_stream;
*/

    stream = gpod_feed_parser_real_download_data_from_url (self, url);

    if (stream == NULL)
        return;
/*
    file = g_file_new_for_uri (local_file_uri)

    if (g_file_query_exists (file))
        file_stream = g_file_open_readwrite ();
    else
        file_stream = g_file_create_readwrite ();
*/
    while (1)
    {
        gchar *out;

        bytes_read = g_input_stream_read (stream,
                                          buffer,
                                          sizeof (buffer),
                                          NULL,
                                          &error);

        /* Stream read finished */
        if (bytes_read == 0)
            break;

        /* Stream read error */
        if (bytes_read < 0)
        {
            g_warning ("Error reading stream chunk, useful debug info:\nURL: \"%s\"\nERROR: \"%s\"", url, error->message);
            g_error_free (error);
            return;
        }

        /* Only read the new bytes, we don't want buffer data from the last dump */
        out = g_strndup (buffer, bytes_read);
        g_print (out);
    }

    retval = g_input_stream_close (stream, NULL, &error);

    if (retval == FALSE)
    {
        g_warning ("Unable to close stream read, useful debug info:\nURL: \"%s\"\nERROR: \"%s\"", url, error->message);
        g_error_free (error);
        return;
    }
}

GpodFeed *
gpod_feed_parser_real_parse_xml_from_url (GpodFeedParser *self,
                                          gchar          *url)
{
    g_assert (GPOD_IS_FEED_PARSER (self));
    g_return_val_if_fail (url != NULL, NULL);

    GInputStream *feed_data;
    GpodFeed *feed;

    feed_data = gpod_feed_parser_real_download_data_from_url (self, url);

    if (feed_data == NULL)
    {
        g_warning ("No feed data to parse from url: \"%s\"", url);
        return NULL;
    }

    feed = gpod_feed_new ();


    //g_input_stream_read (stream, buffer, sizeof buffer, NULL, NULL);


    return feed;
}
